@@ -25,6 +25,7 @@ except ImportError:
 # Local imports
 from .extractor import create_extractor
 from .languages import get_language_for_file
+from .file_reader import get_file_content
 
 
 # Language mapping for file extensions
@@ -127,9 +128,15 @@ LANG_MAP = {
 }
 
 
-def get_language_for_file(file_path: str) -> str:
-    """Get the language name for a file."""
-    ext = Path(file_path).suffix.lower()
+def get_language_for_file(path_or_url: str) -> str:
+    """Get the language name for a file or URL."""
+    # For URLs, extract file extension from the path component
+    if path_or_url.startswith(('http://', 'https://')):
+        from urllib.parse import urlparse
+        parsed = urlparse(path_or_url)
+        ext = Path(parsed.path).suffix.lower()
+    else:
+        ext = Path(path_or_url).suffix.lower()
     return LANG_MAP.get(ext, 'text')
 
 
@@ -142,13 +149,10 @@ def find_function(node) -> dict:
     If you're looking for a specific function, this is better than searching.
     """
     
-    def get_function(file_path: str, function_name: str) -> dict:
+    def get_function(path_or_url: str, function_name: str, git_revision: Optional[str] = None) -> dict:
         """Extract a specific function from a file."""
-        if not os.path.exists(file_path):
-            return {"error": f"File not found: {file_path}"}
-        
         try:
-            lang_name = get_language_for_file(file_path)
+            lang_name = get_language_for_file(path_or_url)
             
             # Get tree-sitter parser
             try:
@@ -156,10 +160,10 @@ def find_function(node) -> dict:
             except Exception:
                 return {"error": f"Language '{lang_name}' not supported"}
             
-            with open(file_path, 'r', encoding='utf-8') as f:
-                source = f.read()
+            source = get_file_content(path_or_url, git_revision)
+            source_bytes = source.encode('utf-8') if isinstance(source, str) else source
             
-            tree = parser.parse(source)
+            tree = parser.parse(source_bytes)
             
             # Define function node types for different languages
             func_types = {
@@ -186,14 +190,18 @@ def find_function(node) -> dict:
                     name = None
                     for child in node.children:
                         if child.type == 'identifier':
-                            name = source[child.start_byte:child.end_byte].decode(
-                                'utf-8') if isinstance(source, bytes) else source[child.start_byte:child.end_byte]
+                            if isinstance(source, str):
+                                name = source[child.start_byte:child.end_byte]
+                            else:
+                                name = source[child.start_byte:child.end_byte].decode('utf-8') if isinstance(source, bytes) else source[child.start_byte:child.end_byte]
                             break
                         elif hasattr(child, 'children'):
                             for grandchild in child.children:
                                 if grandchild.type == 'identifier':
-                                    name = source[grandchild.start_byte:grandchild.end_byte].decode(
-                                        'utf-8') if isinstance(source, bytes) else source[grandchild.start_byte:grandchild.end_byte]
+                                    if isinstance(source, str):
+                                        name = source[grandchild.start_byte:grandchild.end_byte]
+                                    else:
+                                        name = source[grandchild.start_byte:grandchild.end_byte].decode('utf-8') if isinstance(source, bytes) else source[grandchild.start_byte:grandchild.end_byte]
                                     break
                             if name:
                                 break
@@ -201,8 +209,10 @@ def find_function(node) -> dict:
                     # Use field name if available (more reliable)
                     name_node = node.child_by_field_name('name')
                     if name_node:
-                        name = source[name_node.start_byte:name_node.end_byte].decode(
-                            'utf-8') if isinstance(source, bytes) else source[name_node.start_byte:name_node.end_byte]
+                        if isinstance(source, str):
+                            name = source[name_node.start_byte:name_node.end_byte]
+                        else:
+                            name = source[name_node.start_byte:name_node.end_byte].decode('utf-8') if isinstance(source, bytes) else source[name_node.start_byte:name_node.end_byte]
                     
                     if name == function_name:
                         return node
@@ -230,12 +240,12 @@ def find_function(node) -> dict:
                 "end_line": end_line,
                 "lines": f"{start_line}-{end_line}",
                 "function": function_name,
-                "file": file_path,
+                "file": path_or_url,
                 "language": lang_name
             }
             
         except Exception as e:
-            return {"error": f"Failed to parse '{file_path}': {str(e)}"}
+            return {"error": f"Failed to parse '{path_or_url}': {str(e)}"}
     
     return get_function
 
@@ -249,13 +259,10 @@ def find_class(node) -> dict:
     If you're looking for a specific class, this is better than searching.
     """
     
-    def get_class(file_path: str, class_name: str) -> dict:
+    def get_class(path_or_url: str, class_name: str, git_revision: Optional[str] = None) -> dict:
         """Extract a specific class from a file."""
-        if not os.path.exists(file_path):
-            return {"error": f"File not found: {file_path}"}
-        
         try:
-            lang_name = get_language_for_file(file_path)
+            lang_name = get_language_for_file(path_or_url)
             
             # Get tree-sitter parser
             try:
@@ -263,10 +270,10 @@ def find_class(node) -> dict:
             except Exception:
                 return {"error": f"Language '{lang_name}' not supported"}
             
-            with open(file_path, 'r', encoding='utf-8') as f:
-                source = f.read()
+            source = get_file_content(path_or_url, git_revision)
+            source_bytes = source.encode('utf-8') if isinstance(source, str) else source
             
-            tree = parser.parse(source)
+            tree = parser.parse(source_bytes)
             
             # Define class node types for different languages
             class_types = {
@@ -294,16 +301,20 @@ def find_class(node) -> dict:
                     # Extract class name
                     for child in node.children:
                         if child.type == 'identifier':
-                            name = source[child.start_byte:child.end_byte].decode(
-                                'utf-8') if isinstance(source, bytes) else source[child.start_byte:child.end_byte]
+                            if isinstance(source, str):
+                                name = source[child.start_byte:child.end_byte]
+                            else:
+                                name = source[child.start_byte:child.end_byte].decode('utf-8') if isinstance(source, bytes) else source[child.start_byte:child.end_byte]
                             if name == class_name:
                                 return node
                     
                     # Use field name if available (more reliable)
                     name_node = node.child_by_field_name('name')
                     if name_node:
-                        name = source[name_node.start_byte:name_node.end_byte].decode(
-                            'utf-8') if isinstance(source, bytes) else source[name_node.start_byte:name_node.end_byte]
+                        if isinstance(source, str):
+                            name = source[name_node.start_byte:name_node.end_byte]
+                        else:
+                            name = source[name_node.start_byte:name_node.end_byte].decode('utf-8') if isinstance(source, bytes) else source[name_node.start_byte:name_node.end_byte]
                         if name == class_name:
                             return node
                 
@@ -330,17 +341,17 @@ def find_class(node) -> dict:
                 "end_line": end_line,
                 "lines": f"{start_line}-{end_line}",
                 "class": class_name,
-                "file": file_path,
+                "file": path_or_url,
                 "language": lang_name
             }
             
         except Exception as e:
-            return {"error": f"Failed to parse '{file_path}': {str(e)}"}
+            return {"error": f"Failed to parse '{path_or_url}': {str(e)}"}
     
     return get_class
 
 
-def get_symbols(file_path: str) -> list:
+def get_symbols(path_or_url: str, git_revision: Optional[str] = None) -> list:
     """
     🚨 **ALWAYS USE THIS FIRST** for code investigation - DO NOT use Read()!
     
@@ -349,15 +360,9 @@ def get_symbols(file_path: str) -> list:
     DON'T read entire files to understand code structure - use this instead.
     """
     
-    if not os.path.exists(file_path):
-        return [{"error": f"File not found: {file_path}"}]
-    
     try:
-        extractor = create_extractor(file_path)
-        
-        with open(file_path, 'r', encoding='utf-8') as f:
-            source_code = f.read()
-        
+        extractor = create_extractor(path_or_url)
+        source_code = get_file_content(path_or_url, git_revision)
         symbols = extractor.extract_symbols(source_code)
         
         # Convert to dict format for MCP compatibility
@@ -368,19 +373,16 @@ def get_symbols(file_path: str) -> list:
         return result
         
     except Exception as e:
-        return [{"error": f"Failed to parse '{file_path}': {str(e)}"}]
+        return [{"error": f"Failed to parse '{path_or_url}': {str(e)}"}]
 
 
-def get_lines(file_path: str, start_line: int, end_line: int) -> dict:
+def get_lines(path_or_url: str, start_line: int, end_line: int, git_revision: Optional[str] = None) -> dict:
     """
     Get specific lines from a file using precise line range control.
     
     Use when you know exact line numbers - better than reading entire files.
     """
     
-    if not os.path.exists(file_path):
-        return {"error": f"File not found: {file_path}"}
-    
     try:
         if start_line < 1:
             return {"error": "start_line must be >= 1"}
@@ -388,8 +390,8 @@ def get_lines(file_path: str, start_line: int, end_line: int) -> dict:
         if end_line < start_line:
             return {"error": "end_line must be >= start_line"}
         
-        with open(file_path, 'r', encoding='utf-8') as f:
-            lines = f.readlines()
+        source_code = get_file_content(path_or_url, git_revision)
+        lines = source_code.splitlines(keepends=True)
         
         # Convert to 0-based indexing
         start_idx = max(0, start_line - 1)
@@ -402,21 +404,21 @@ def get_lines(file_path: str, start_line: int, end_line: int) -> dict:
             "start_line": start_line,
             "end_line": min(end_line, len(lines)),
             "lines": f"{start_line}-{min(end_line, len(lines))}",
-            "file": file_path
+            "file": path_or_url
         }
         
     except Exception as e:
-        return {"error": f"Failed to read '{file_path}': {str(e)}"}
+        return {"error": f"Failed to read '{path_or_url}': {str(e)}"}
 
 
-def get_signature(file_path: str, function_name: str) -> dict:
+def get_signature(path_or_url: str, function_name: str, git_revision: Optional[str] = None) -> dict:
     """
     Get just the signature/declaration of a function without full implementation.
     
     Use for function interfaces, parameters, return types. Lighter than get_function.
     """
     
-    result = find_function(None)(file_path, function_name)
+    result = find_function(None)(path_or_url, function_name, git_revision)
     
     if "error" in result:
         return result
@@ -428,7 +430,7 @@ def get_signature(file_path: str, function_name: str) -> dict:
     return {
         "signature": signature,
         "function": function_name,
-        "file": file_path,
+        "file": path_or_url,
         "start_line": result["start_line"]
     }
 
@@ -439,55 +441,101 @@ def main():
     mcp = FastMCP("extract")
     
     @mcp.tool()
-    def get_symbols_tool(file_path: str) -> list:
+    def get_symbols_tool(path_or_url: str, git_revision: Optional[str] = None) -> list:
         """
         🚨 **ALWAYS USE THIS FIRST** for code investigation - DO NOT use Read()!
         
         List all functions, classes, and symbols with line numbers.
         
         DON'T read entire files to understand code structure - use this instead.
+        
+        Args:
+            path_or_url: Path to source file or URL (GitHub raw, GitLab raw, direct file URL)
+            git_revision: Optional git revision (commit, branch, tag, HEAD~1, etc.) - not supported for URLs
+            
+        Examples:
+            get_symbols_tool("/path/to/file.py")  # Local file
+            get_symbols_tool("https://raw.githubusercontent.com/user/repo/main/file.py")  # GitHub raw URL
+            get_symbols_tool("/path/to/file.py", "HEAD~1")  # Git revision
         """
-        return get_symbols(file_path)
+        return get_symbols(path_or_url, git_revision)
     
     @mcp.tool()
-    def get_function_tool(file_path: str, function_name: str) -> dict:
+    def get_function_tool(path_or_url: str, function_name: str, git_revision: Optional[str] = None) -> dict:
         """
         Extract function definition - USE THIS INSTEAD OF Read() for specific functions!
         
         DON'T use Read() + grep/search. Use this for precise extraction with tree-sitter.
         
         If you're looking for a specific function, this is better than searching.
+        
+        Args:
+            path_or_url: Path to source file or URL (GitHub raw, GitLab raw, direct file URL)
+            function_name: Name of the function to extract
+            git_revision: Optional git revision (commit, branch, tag, HEAD~1, etc.) - not supported for URLs
+            
+        Examples:
+            get_function_tool("/path/to/file.py", "my_function")  # Local file
+            get_function_tool("https://raw.githubusercontent.com/user/repo/main/file.py", "my_function")  # GitHub raw URL
         """
-        return find_function(None)(file_path, function_name)
+        return find_function(None)(path_or_url, function_name, git_revision)
     
     @mcp.tool()
-    def get_class_tool(file_path: str, class_name: str) -> dict:
+    def get_class_tool(path_or_url: str, class_name: str, git_revision: Optional[str] = None) -> dict:
         """
         Extract class definition - USE THIS INSTEAD OF Read() for specific classes!
         
         DON'T use Read() + grep/search. Use this for precise extraction with tree-sitter.
         
         If you're looking for a specific class, this is better than searching.
+        
+        Args:
+            path_or_url: Path to source file or URL (GitHub raw, GitLab raw, direct file URL)
+            class_name: Name of the class to extract
+            git_revision: Optional git revision (commit, branch, tag, HEAD~1, etc.) - not supported for URLs
+            
+        Examples:
+            get_class_tool("/path/to/file.py", "MyClass")  # Local file
+            get_class_tool("https://raw.githubusercontent.com/user/repo/main/file.py", "MyClass")  # GitHub raw URL
         """
-        return find_class(None)(file_path, class_name)
+        return find_class(None)(path_or_url, class_name, git_revision)
     
     @mcp.tool()
-    def get_lines_tool(file_path: str, start_line: int, end_line: int) -> dict:
+    def get_lines_tool(path_or_url: str, start_line: int, end_line: int, git_revision: Optional[str] = None) -> dict:
         """
         Get specific lines from a file using precise line range control.
         
         Use when you know exact line numbers - better than reading entire files.
+        
+        Args:
+            path_or_url: Path to source file or URL (GitHub raw, GitLab raw, direct file URL)
+            start_line: Starting line number (1-based)
+            end_line: Ending line number (1-based, inclusive)
+            git_revision: Optional git revision (commit, branch, tag, HEAD~1, etc.) - not supported for URLs
+            
+        Examples:
+            get_lines_tool("/path/to/file.py", 10, 20)  # Local file lines 10-20
+            get_lines_tool("https://raw.githubusercontent.com/user/repo/main/file.py", 1, 50)  # GitHub raw URL
         """
-        return get_lines(file_path, start_line, end_line)
+        return get_lines(path_or_url, start_line, end_line, git_revision)
     
     @mcp.tool()
-    def get_signature_tool(file_path: str, function_name: str) -> dict:
+    def get_signature_tool(path_or_url: str, function_name: str, git_revision: Optional[str] = None) -> dict:
         """
         Get just the signature/declaration of a function without full implementation.
         
         Use for function interfaces, parameters, return types. Lighter than get_function.
+        
+        Args:
+            path_or_url: Path to source file or URL (GitHub raw, GitLab raw, direct file URL)
+            function_name: Name of the function to get signature for
+            git_revision: Optional git revision (commit, branch, tag, HEAD~1, etc.) - not supported for URLs
+            
+        Examples:
+            get_signature_tool("/path/to/file.py", "my_function")  # Local file
+            get_signature_tool("https://raw.githubusercontent.com/user/repo/main/file.py", "my_function")  # GitHub raw URL
         """
-        return get_signature(file_path, function_name)
+        return get_signature(path_or_url, function_name, git_revision)
     
     # Run the server
     mcp.run()
